local arrow
local arrowHalfW, arrowHalfH
local zoom = 1

local playerX, playerY, playerW, playerH = 100, 200, 20, 40
local enemyX, enemyY, enemyW, enemyH = 300, 400, 20, 40
local isEnemyOff = false
local arrowPosX, arrowPosY

function love.load()
  print(0/0 > 1)
  arrow = love.graphics.newImage('small-arrow.png')
  arrowHalfW = arrow:getWidth() / 2
  arrowHalfH = arrow:getHeight() / 2
end

function love.wheelmoved(x, y)
  zoom = zoom * 1.1 ^ y
  if zoom > 1 then zoom = 1 end
end

function love.update(dt)
  isEnemyOff = enemyX < 0 or enemyX >= love.graphics.getWidth()
            or enemyY < 0 or enemyY >= love.graphics.getHeight()
  if isEnemyOff then
    local screenW, screenH = love.graphics.getDimensions()
    local vx, vy = enemyX - playerX, enemyY - playerY
    local t, minT
    minT = math.huge
    t = (arrowHalfW - playerX) / vx
    if t == t and t > 0 and t < minT then
      minT = t
      arrowPosX = arrowHalfW
      arrowPosY = vy * t + playerY
    end
    t = (arrowHalfH - playerY) / vy
    if t == t and t > 0 and t < minT then
      minT = t
      arrowPosX = vx * t + playerX
      arrowPosY = arrowHalfH
    end
    t = (screenW - arrowHalfW - playerX) / vx
    if t == t and t > 0 and t < minT then
      minT = t
      arrowPosX = screenW - arrowHalfW
      arrowPosY = vy * t + playerY
    end
    t = (screenH - arrowHalfH - playerY) / vy
    if t > 0 and t < minT then
      -- minT = t -- doesn't matter anymore
      arrowPosX = vx * t + playerX
      arrowPosY = screenH - arrowHalfH
    end
  end
end

function love.draw()
  local screenW, screenH = love.graphics.getDimensions()
  love.graphics.translate(screenW/2, screenH/2)
  love.graphics.scale(zoom)
  love.graphics.translate(screenW/-2, screenH/-2)

  -- Draw screen border
  love.graphics.setLineWidth(1/zoom)
  love.graphics.rectangle("line", -.5, -.5, screenW+1, screenH+1)

  -- Draw player and enemy
  love.graphics.setColor(0, 1, 1)
  love.graphics.rectangle("fill", playerX - playerW/2, playerY - playerH/2, playerW, playerH)
  love.graphics.setColor(1, 0, 0)
  love.graphics.rectangle("fill", enemyX - enemyW/2, enemyY - enemyH/2, enemyW, enemyH)

  -- Draw arrow if visible
  if isEnemyOff then
    love.graphics.setColor(1, 1, 0)
    love.graphics.draw(arrow, arrowPosX, arrowPosY,
      math.atan2(enemyY - playerY, enemyX - playerX),
      1, 1, arrowHalfW, arrowHalfH)
  end

  -- Restore colour
  love.graphics.setColor(1, 1, 1)
end

function love.mousemoved(x, y)
  -- transform screen to zoomed coordinates
  x = (x - love.graphics.getWidth() / 2) / zoom + love.graphics.getWidth() / 2
  y = (y - love.graphics.getHeight() / 2) / zoom + love.graphics.getHeight() / 2

  if love.mouse.isDown(1) then
    playerX = x
    playerY = y
  elseif love.mouse.isDown(2) then
    enemyX = x
    enemyY = y
  end
end
love.mousepressed = love.mousemoved
love.mousereleased = love.mousemoved
